@@ -28,7 +28,7 @@ fn main() {
 
     let curdir_str = match env::current_dir() {
         Ok(p) => p.to_str().unwrap_or("").to_string(),
-        Err(_) => "".to_string(),
+        Err(_) => "".to_string()
     };
     let has_cratesfyi = curdir_str.contains("cratesfyi");
 
@@ -40,7 +40,7 @@ fn main() {
 
     let file_exists = match fs::metadata(&pbuf_rs_file) {
         Ok(f) => f.is_file(),
-        Err(_) => false,
+        Err(_) => false
     };
 
     // None of this shows up in the logs unless there is a panic
@@ -50,9 +50,9 @@ fn main() {
     println!("systemctl.rs exists:  {:?}", file_exists);
 
     let args = protoc_rust::Args {
-            out_dir: "src",
-            input: &["src/sysadminctl.proto"],
-            includes: &["src"],
+        out_dir: "src",
+        input: &["src/sysadminctl.proto"],
+        includes: &["src"]
     };
 
     let result = panic::catch_unwind(|| {
@@ -60,8 +60,10 @@ fn main() {
     });
 
     if !(has_cratesfyi && file_exists) {
-        println!("This doesn't look like \"cargo doc\" so any \
-            panics will be unwound.");
+        println!(
+            "This doesn't look like \"cargo doc\" so any \
+             panics will be unwound."
+        );
         if let Err(err) = result {
             println!("Panics encountered!");
             panic::resume_unwind(err);
